/*
 * An access map providing memory corresponding to file regions.
 *
 * Copyright (C) 2021 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#pragma once

#include <mem/flexpage.h>
#include <fsserver/page_owner.h>
#include <fsserver/pages.h>

#include <map>
#include <mutex>



/* Collection types. */

typedef std::map<offset_t, Flexpage *> _AccessMap;
typedef std::pair<offset_t, Flexpage *> _AccessMapEntry;



/* A mapping from file positions to flexpages. */

class AccessMap
{
protected:
    _AccessMap _flexpages;
    std::mutex _lock;

public:
    Flexpage *find(offset_t position);

    void insert(Flexpage *flexpage);

    bool remove(PageOwner *owner, Flexpage *flexpage);

    void purge(PageOwner *owner, Pages *pages);

    void flush_all(offset_t start, offset_t size, PageOwner *owner, Pages *pages);
};

// vim: tabstop=4 expandtab shiftwidth=4
